""" InfluxDB Query Object. """
import functools

from . import meta


class InfluxDBQuery(object):
    """ InfluxDB Query object.

        entities    (tuple):          Query entities
        client      (InfluxAlchemy):  InfluxAlchemy instance
        expressions (tuple):          Query filters
        groupby     (str):            GROUP BY string
        limit       (int):            LIMIT int
    """
    def __init__(self, entities, client, expressions=None, groupby=None,
                 limit=None):
        # pylint: disable=too-many-arguments
        self._entities = entities
        self._client = client
        self._expressions = expressions or ()
        self._groupby = groupby
        self._limit = limit

    def __str__(self):
        select = ", ".join(self._select)
        from_ = self._from
        where = " AND ".join(self._where)
        if any(where):
            iql = "SELECT %s FROM %s WHERE %s" % (select, from_, where)
        else:
            iql = "SELECT %s FROM %s" % (select, from_)
        if self._groupby is not None:
            iql += " GROUP BY %s" % self._groupby
        if self._limit is not None:
            iql += " LIMIT {0}".format(self._limit)
        return "%s;" % iql

    def __repr__(self):
        return str(self)

    def execute(self):
        """ Execute query. """
        return self._client.bind.query(str(self))

    def filter(self, *expressions):
        """ Filter query. """
        expressions = self._expressions + expressions
        return InfluxDBQuery(self._entities, self._client,
                             expressions=expressions)

    def filter_by(self, **kwargs):
        """ Filter query by tag value. """
        expressions = self._expressions
        for key, val in sorted(kwargs.items()):
            expressions += (meta.TagExp.equals(key, val),)
        return InfluxDBQuery(self._entities, self._client,
                             expressions=expressions)

    def group_by(self, groupby):
        """ Group query. """
        return InfluxDBQuery(
            self._entities, self._client, self._expressions, groupby)

    def limit(self, limit):
        """ Limit query """
        assert isinstance(limit, int)
        return InfluxDBQuery(
            self._entities, self._client, self._expressions, self._groupby,
            limit)

    @property
    def measurement(self):
        """ Query measurement. """
        measurements = set(x.measurement for x in self._entities)
        return functools.reduce(lambda x, y: x | y, measurements)

    @property
    def _select(self):
        """ SELECT statement. """
        selects = []
        for ent in self._entities:
            # Entity is a Tag
            if isinstance(ent, meta.Tag):
                selects.append(str(ent))
            # Entity is a Measurement
            else:
                try:
                    for tag in self._client.tags(ent):
                        selects.append(tag)
                    for field in self._client.fields(ent):
                        selects.append(field)
                # pylint: disable=broad-except
                except Exception:
                    pass
        return selects or ["*"]

    @property
    def _from(self):
        """ FROM statement. """
        return str(self.measurement)

    @property
    def _where(self):
        """ WHERE statement. """
        for exp in self._expressions:
            yield "(%s)" % exp



""" InfluxAlchemy Operations. """

from influxalchemy.operations import Operation


def test_op_init():
    op = Operation(" fizz ", " buzz ")
    assert op._op == " fizz "
    assert op._nop == " buzz "


def test_op_str():
    op = Operation(" fizz ", " buzz ")
    assert str(op) == " fizz "


def test_op_repr():
    op = Operation(" fizz ", " buzz ")
    assert repr(op) == " fizz "


def test_op_inv():
    op = ~Operation(" fizz ", " buzz ")
    assert op._nop == " fizz "
    assert op._op == " buzz "


def test_op_eq():
    op0 = Operation(" fizz ", " buzz ")
    op1 = Operation(" fizz ", " buzz ")
    assert op0 == op1


def test_op_ne():
    op0 = Operation(" fizz ", " buzz ")
    op1 = ~op0
    assert op0 != op1



""" InfluxAlchemy Client. """
from . import query
from .measurement import Measurement


class InfluxAlchemy(object):
    """ InfluxAlchemy database session.

        client (InfluxDBClient):  Connection to InfluxDB database
    """
    def __init__(self, client):
        self.bind = client
        # pylint: disable=protected-access
        assert self.bind._database is not None, \
            "InfluxDB client database cannot be None"

    def query(self, *entities):
        """ Query InfluxDB entities. Entities are either Measurements or
            Tags/Fields.
        """
        return query.InfluxDBQuery(entities, self)

    def measurements(self):
        """ Get measurements of an InfluxDB. """
        results = self.bind.query("SHOW MEASUREMENTS;")
        for res in results.get_points():
            yield Measurement.new(str(res["name"]))

    def tags(self, measurement):
        """ Get tags of a measurements in InfluxDB. """
        tags = self.bind.query("SHOW tag keys FROM %s" % measurement)
        pts = sorted(set(t['tagKey'] for t in tags.get_points()))
        return pts

    def fields(self, measurement):
        """ Get fields of a measurements in InfluxDB. """
        fields = self.bind.query("SHOW field keys FROM %s" % measurement)
        pts = sorted(set(f['fieldKey'] for f in fields.get_points()))
        return pts



""" InfluxDB Meta Measurement. """
from datetime import date

from . import operations

try:
    from datetime import timezone
    UTC = timezone.utc  # pragma: no cover
except ImportError:     # pragma: no cover
    import pytz         # pragma: no cover
    UTC = pytz.utc      # pragma: no cover


def make_tz_aware(datetime_obj):
    """ Make a date/datetime object to be timezone-aware """
    # 'date' object doesn't need to be timezone aware
    # pylint: disable=unidiomatic-typecheck
    if type(datetime_obj) is date:
        return datetime_obj
    # Already aware
    if datetime_obj.tzinfo:
        return datetime_obj
    # With naive datetime object, assume it is UTC
    return datetime_obj.replace(tzinfo=UTC)


class MetaMeasurement(type):
    """ Meta class of Measurement. """
    def __new__(cls, name, bases, dict_):
        dict_.setdefault("__measurement__", name)
        return super(MetaMeasurement, cls).__new__(cls, name, bases, dict_)

    def __getattribute__(cls, name):
        try:
            return super(MetaMeasurement, cls).__getattribute__(name)
        except AttributeError:
            if name == "time":
                return Time(name, cls)
            return Tag(name, cls)

    def __str__(cls):
        return cls.__measurement__

    def __eq__(cls, other):
        return str(cls) == str(other)

    def __ne__(cls, other):
        return str(cls) != str(other)

    def __or__(cls, other):
        left = str(cls).strip("/")
        name = "_".join(left.split("|") + [str(other)])
        bases = (cls,)
        measurement = "/%s|%s/" % (str(cls).strip("/"), other)
        return type(name, bases, {"__measurement__": measurement})

    def __hash__(cls):
        return id(cls)

    @property
    def measurement(cls):
        """ Reflexive reference to measurement. """
        return cls


class Tag(object):
    """ InfluxDB Tag instance.

        name        (str):          Name of Tag
        measurement (Measurement):  Measurement of tag
    """
    def __init__(self, name, measurement):
        self._name = name
        self.measurement = measurement

    def __str__(self):
        return self._name

    def __repr__(self):
        return "<%s.%s>" % (self.measurement.__measurement__, self._name)

    def __eq__(self, other):
        return TagExp.equals(self, other)

    def __ne__(self, other):
        return TagExp.notequals(self, other)

    def __gt__(self, other):
        return TagExp.greater_than(self, other)

    def __lt__(self, other):
        return TagExp.less_than(self, other)

    def __ge__(self, other):
        return TagExp.greater_equal(self, other)

    def __le__(self, other):
        return TagExp.less_equal(self, other)

    def like(self, other):
        """ self =~ other """
        return TagExp.like(self, other)

    def notlike(self, other):
        """ self !~ other """
        return TagExp.notlike(self, other)


class Time(Tag):
    """ Time of InfluxDB Measurement. """
    def between(self, start, end, startinc=True, endinc=True):
        """ Query times between extremes.

            Arguments:
                start    (str):      Start of time
                end      (str):      End of time
                startinc (boolean):  Start-inclusive flag
                endinc   (boolean):  End-inclusive flag

            Returns:
                Time expression.
        """
        if startinc is True:
            startexp = TagExp.greater_equal(self, start)
        else:
            startexp = TagExp.greater_than(self, start)
        if endinc is True:
            endexp = TagExp.less_equal(self, end)
        else:
            endexp = TagExp.less_than(self, end)
        return startexp & endexp


class TagExp(object):
    """ A tag query expression. """
    def __init__(self, left, op, right):
        self._left = left
        self._op = op
        lits = [operations.LK, operations.NK, operations.AND, operations.OR]
        if self._op in lits or isinstance(left, Time):
            # If the right-hand-side value is a date/datetime object,
            # we convert it to RFC3339 representation
            # and wrap inside single quote
            if isinstance(right, date):
                right = repr(make_tz_aware(right).isoformat())
            self._right = right
        else:
            self._right = repr(right)

    def __str__(self):
        return "%s%s%s" % (self._left, self._op, self._right)

    def __repr__(self):
        return "[ %s ]" % self

    def __eq__(self, other):
        return str(self) == str(other)

    def __ne__(self, other):
        return str(self) != str(other)

    def __and__(self, other):
        return TagExp(str(self), " AND ", str(other))

    def __or__(self, other):
        return TagExp(str(self), " OR ", str(other))

    def __invert__(self):
        return TagExp(self._left, ~self._op, self._right)

    @classmethod
    def equals(cls, self, other):
        """ left = right """
        return cls(self, operations.EQ, other)

    @classmethod
    def notequals(cls, self, other):
        """ left != right """
        return cls(self, operations.NE, other)

    @classmethod
    def greater_than(cls, self, other):
        """ left > right """
        return cls(self, operations.GT, other)

    @classmethod
    def less_than(cls, self, other):
        """ left < right """
        return cls(self, operations.LT, other)

    @classmethod
    def greater_equal(cls, self, other):
        """ left >= right """
        return cls(self, operations.GE, other)

    @classmethod
    def less_equal(cls, self, other):
        """ left <= right """
        return cls(self, operations.LE, other)

    @classmethod
    def like(cls, self, other):
        """ left =~ right """
        return cls(self, operations.LK, other)

    @classmethod
    def notlike(cls, self, other):
        """ left !~ right """
        return cls(self, operations.NK, other)



""" InfluxDB Operations. """


# pylint: disable=too-few-public-methods
class Operation(object):
    """ InfluxDB query operation. """
    def __init__(self, op, nop):
        self._op = op
        self._nop = nop

    def __str__(self):
        return self._op

    def __repr__(self):
        return str(self)

    def __invert__(self):
        return Operation(self._nop, self._op)

    def __eq__(self, other):
        return str(self) == str(other)

    def __ne__(self, other):
        return str(self) != str(other)


EQ = Operation(" = ", " != ")
NE = Operation(" != ", " = ")
GT = Operation(" > ", " <= ")
LT = Operation(" < ", " >= ")
GE = Operation(" >= ", " < ")
LE = Operation(" <= ", " > ")
LK = Operation(" =~ ", " !~ ")
NK = Operation(" !~ ", " =~ ")
AND = Operation(" AND ", " OR ")
OR = Operation(" OR ", " AND ")



""" InfluxDB Measurement. """
import six
from . import meta


# pylint: disable=too-few-public-methods
class Measurement(six.with_metaclass(meta.MetaMeasurement)):
    """ InfluxDB Measurement. """
    # pylint: disable=no-init
    @classmethod
    def new(cls, name):
        """ Generate new Measurement class. """
        return type(name, (cls,), {"__measurement__": name})



""" InfluxAlchemy Query Tests. """
import sys
from datetime import date, datetime, timedelta
try:
    from datetime import timezone
except ImportError:
    from pytz import timezone

import influxdb
import mock
from influxalchemy.client import InfluxAlchemy
from influxalchemy.measurement import Measurement


@mock.patch("influxdb.InfluxDBClient.query")
def test_repr(mock_qry):
    mock_qry.side_effect = influxdb.exceptions.InfluxDBClientError(None)
    db = influxdb.InfluxDBClient(database="example")
    client = InfluxAlchemy(db)
    query = client.query(Measurement.new("fizz"))
    assert repr(query) == "SELECT * FROM fizz;"


@mock.patch("influxdb.InfluxDBClient.query")
def test_execute(mock_qry):
    db = influxdb.InfluxDBClient(database="example")
    client = InfluxAlchemy(db)
    query = client.query(Measurement.new("fizz").buzz)
    query.execute()
    mock_qry.assert_called_with("SELECT buzz FROM fizz;")


@mock.patch("influxdb.InfluxDBClient.query")
def test_filter(mock_qry):
    mock_qry.side_effect = influxdb.exceptions.InfluxDBClientError(None)
    db = influxdb.InfluxDBClient(database="example")
    client = InfluxAlchemy(db)
    meas = Measurement.new("fizz")
    query = client.query(meas).filter(meas.buzz == "goo")
    assert repr(query) == "SELECT * FROM fizz WHERE (buzz = 'goo');"


@mock.patch("influxdb.InfluxDBClient.query")
def test_filter_time_naive(mock_qry):
    mock_qry.side_effect = influxdb.exceptions.InfluxDBClientError(None)
    db = influxdb.InfluxDBClient(database="example")
    client = InfluxAlchemy(db)
    meas = Measurement.new("fizz")
    d = datetime(2016, 10, 1)
    query = client.query(meas).filter(meas.time >= d)
    assert repr(query) == \
        "SELECT * FROM fizz WHERE (time >= '2016-10-01T00:00:00+00:00');"


@mock.patch("influxdb.InfluxDBClient.query")
def test_filter_time_date(mock_qry):
    mock_qry.side_effect = influxdb.exceptions.InfluxDBClientError(None)
    db = influxdb.InfluxDBClient(database="example")
    client = InfluxAlchemy(db)
    meas = Measurement.new("fizz")
    d = date(2016, 10, 1)
    query = client.query(meas).filter(meas.time >= d)
    assert repr(query) == "SELECT * FROM fizz WHERE (time >= '2016-10-01');"


@mock.patch("influxdb.InfluxDBClient.query")
def test_filter_time_aware(mock_qry):
    mock_qry.side_effect = influxdb.exceptions.InfluxDBClientError(None)
    db = influxdb.InfluxDBClient(database="example")
    client = InfluxAlchemy(db)
    meas = Measurement.new("fizz")
    if sys.version_info.major >= 3:
        tz_vietnam = timezone(timedelta(hours=7, minutes=7))
    else:
        tz_vietnam = timezone('Asia/Ho_Chi_Minh')
    d_low = datetime(2016, 9, 1, tzinfo=tz_vietnam)
    d_high = datetime(2016, 10, 2, 8)
    query = client.query(meas).filter(meas.time.between(d_low, d_high))
    assert repr(query) == \
        "SELECT * FROM fizz WHERE (time >= '2016-09-01T00:00:00+07:07' "\
        "AND time <= '2016-10-02T08:00:00+00:00');"


@mock.patch("influxdb.InfluxDBClient.query")
def test_group_by(mock_qry):
    mock_qry.side_effect = influxdb.exceptions.InfluxDBClientError(None)
    db = influxdb.InfluxDBClient(database="example")
    client = InfluxAlchemy(db)
    query = client.query(Measurement.new("fizz")).group_by("buzz")
    assert str(query) == "SELECT * FROM fizz GROUP BY buzz;"


@mock.patch("influxdb.InfluxDBClient.query")
def test_filter_by(mock_qry):
    mock_qry.side_effect = influxdb.exceptions.InfluxDBClientError(None)
    db = influxdb.InfluxDBClient(database="example")
    client = InfluxAlchemy(db)
    query = client.query(Measurement.new("fizz")).filter_by(buzz="goo")
    assert str(query) == "SELECT * FROM fizz WHERE (buzz = 'goo');"


def test_tags():
    db = influxdb.InfluxDBClient(database="example")
    client = InfluxAlchemy(db)
    fizz = Measurement.new("fizz")
    query = client.query(fizz.buzz, fizz.bug)
    assert str(query) == "SELECT buzz, bug FROM fizz;"


@mock.patch("influxalchemy.InfluxAlchemy.tags")
@mock.patch("influxalchemy.InfluxAlchemy.fields")
def test_get_tags_fields(mock_fields, mock_tags):
    mock_tags.return_value = ["fizz", "buzz"]
    mock_fields.return_value = ["foo", "goo"]
    db = influxdb.InfluxDBClient(database="example")
    client = InfluxAlchemy(db)
    fizz = Measurement.new("fuzz")
    query = client.query(fizz)
    assert str(query) == "SELECT fizz, buzz, foo, goo FROM fuzz;"


@mock.patch("influxalchemy.InfluxAlchemy.tags")
@mock.patch("influxalchemy.InfluxAlchemy.fields")
def test_get_empty_tags_fields(mock_fields, mock_tags):
    mock_tags.return_value = []
    mock_fields.return_value = []
    db = influxdb.InfluxDBClient(database="example")
    client = InfluxAlchemy(db)
    fizz = Measurement.new("fuzz")
    query = client.query(fizz)
    assert str(query) == "SELECT * FROM fuzz;"


@mock.patch("influxdb.InfluxDBClient.query")
def test_limit_1(mock_limit):
    db = influxdb.InfluxDBClient(database="example")
    client = InfluxAlchemy(db)
    fizz = Measurement.new("fuzz")
    query = client.query(fizz).limit(1)
    assert str(query) == "SELECT * FROM fuzz LIMIT 1;"


@mock.patch("influxdb.InfluxDBClient.query")
def test_limit_2(mock_limit):
    db = influxdb.InfluxDBClient(database="example")
    client = InfluxAlchemy(db)
    fizz = Measurement.new("fuzz")
    query = client.query(fizz).filter_by(vendor='quandl', market='XCME')
    assert str(query) == \
        "SELECT * FROM fuzz WHERE (market = 'XCME') AND (vendor = 'quandl');"


@mock.patch("influxdb.InfluxDBClient.query")
def test_limit_3(mock_limit):
    db = influxdb.InfluxDBClient(database="example")
    client = InfluxAlchemy(db)
    fizz = Measurement.new("fuzz")
    query = client.query(fizz)\
                  .filter(fizz.foo == '123')\
                  .filter(fizz.boo == '555')\
                  .limit(2)
    assert str(query) == \
        "SELECT * FROM fuzz WHERE (foo = '123') AND (boo = '555') LIMIT 2;"



""" InfluxDB Alchemy. """
import pkg_resources

from .client import InfluxAlchemy
from .measurement import Measurement


try:
    __version__ = pkg_resources.get_distribution(__package__).version
except pkg_resources.DistributionNotFound:  # pragma: no cover
    __version__ = None                      # pragma: no cover



""" InfluxAlchemy Measurements. """

from datetime import datetime
from influxalchemy.measurement import Measurement
from influxalchemy.meta import Tag
from influxalchemy.meta import TagExp
from influxalchemy.operations import *


def test_meta_getattr():
    meas = Measurement.new("fizz")
    assert meas.buzz == Tag("buzz", meas)


def test_meta_str():
    meas = Measurement.new("fizz")
    assert str(meas) == "fizz"


def test_meta_ne():
    meas0 = Measurement.new("fizz")
    meas1 = Measurement.new("buzz")
    assert meas0 != meas1


def test_meta_or():
    meas0 = Measurement.new("fizz")
    meas1 = Measurement.new("buzz")
    assert (meas0 | meas1) == Measurement.new("/fizz|buzz/")


def test_meta_measurement():
    meas = Measurement.new("fizz")
    assert meas == meas.measurement


class Fizz(Measurement):
    __measurement__ = "fizz"


def test_new():
    assert Measurement.new("fizz") == Fizz


def test_tag_init():
    meas = Measurement.new("fizz")
    tag = Tag("buzz", meas)
    assert tag == meas.buzz


def test_tag_str():
    meas = Measurement.new("fizz")
    tag = Tag("buzz", meas)
    assert str(tag) == "buzz"


def test_tag_repr():
    meas = Measurement.new("fizz")
    tag = Tag("buzz", meas)
    assert repr(tag) == "<fizz.buzz>"


def test_tag_eq():
    meas = Measurement.new("fizz")
    tag = Tag("buzz", meas)
    exp = tag == "foo"
    assert exp == TagExp("buzz", EQ, "foo")


def test_tag_ne():
    meas = Measurement.new("fizz")
    tag = Tag("buzz", meas)
    exp = tag != "foo"
    assert exp == TagExp("buzz", NE, "foo")


def test_tag_gt():
    meas = Measurement.new("fizz")
    tag = Tag("buzz", meas)
    exp = tag > "foo"
    assert exp == TagExp("buzz", GT, "foo")


def test_tag_lt():
    meas = Measurement.new("fizz")
    tag = Tag("buzz", meas)
    exp = tag < "foo"
    assert exp == TagExp("buzz", LT, "foo")


def test_tag_ge():
    meas = Measurement.new("fizz")
    tag = Tag("buzz", meas)
    exp = tag >= "foo"
    assert exp == TagExp("buzz", GE, "foo")


def test_tag_le():
    meas = Measurement.new("fizz")
    tag = Tag("buzz", meas)
    exp = tag <= "foo"
    assert exp == TagExp("buzz", LE, "foo")


def test_tag_like():
    meas = Measurement.new("fizz")
    tag = Tag("buzz", meas)
    exp = tag.like("foo")
    assert exp == TagExp("buzz", LK, "foo")


def test_tag_notlike():
    meas = Measurement.new("fizz")
    tag = Tag("buzz", meas)
    exp = tag.notlike("foo")
    assert exp == TagExp("buzz", NK, "foo")


def test_time_between():
    meas = Measurement.new("fizz")
    exp = meas.time.between("'2016-01-01'", "now() - 7d")
    assert exp == \
        TagExp(meas.time, " >= ", "'2016-01-01'") & \
        TagExp(meas.time, " <= ", "now() - 7d")


def test_time_between_excl():
    meas = Measurement.new("fizz")
    exp = meas.time.between("'2016-01-01'", "now() - 7d", False, False)
    assert exp == \
        TagExp(meas.time, " > ", "'2016-01-01'") & \
        TagExp(meas.time, " < ", "now() - 7d")


def test_time_between_dt():
    meas = Measurement.new("fizz")
    d = datetime(2016, 1, 1)
    exp = meas.time.between(d, "now() - 7d")
    assert exp == \
        TagExp(meas.time, " >= ", d) & \
        TagExp(meas.time, " <= ", "now() - 7d")


def test_exp_init():
    meas = Measurement.new("fizz")
    exp = TagExp(meas.buzz, " = ", "goo")
    assert exp._left == meas.buzz
    assert exp._op == " = "
    assert exp._right == "'goo'"


def test_exp_str():
    meas = Measurement.new("fizz")
    exp = TagExp(meas.buzz, " = ", "goo")
    assert str(exp) == "buzz = 'goo'"


def test_exp_repr():
    meas = Measurement.new("fizz")
    exp = TagExp(meas.buzz, " = ", "goo")
    assert repr(exp) == "[ buzz = 'goo' ]"


def test_exp_ne():
    meas = Measurement.new("fizz")
    exp0 = TagExp(meas.buzz, " = ", "goo")
    exp1 = TagExp(meas.guzz, " = ", "zoo")
    assert exp0 != exp1


def test_exp_and():
    meas = Measurement.new("fizz")
    exp0 = TagExp(meas.buzz, " = ", "goo")
    exp1 = TagExp(meas.guzz, " = ", "zoo")
    assert (exp0 & exp1) == \
        TagExp("buzz = 'goo'", " AND ", "guzz = 'zoo'")


def test_exp_or():
    meas = Measurement.new("fizz")
    exp0 = TagExp(meas.buzz, " = ", "goo")
    exp1 = TagExp(meas.guzz, " = ", "zoo")
    assert (exp0 | exp1) == \
        TagExp("buzz = 'goo'", " OR ", "guzz = 'zoo'")


def test_exp_inv():
    meas = Measurement.new("fizz")
    exp = TagExp(meas.buzz, EQ, "goo")
    assert ~exp == TagExp(meas.buzz, NE, "'goo'")


def test_equals():
    meas = Measurement.new("fizz")
    exp = TagExp.equals(meas.buzz, "goo")
    assert exp == TagExp(meas.buzz, EQ, "goo")


def test_notequals():
    meas = Measurement.new("fizz")
    exp = TagExp.notequals(meas.buzz, "goo")
    assert exp == TagExp(meas.buzz, NE, "goo")


def test_greater_than():
    meas = Measurement.new("fizz")
    exp = TagExp.greater_than(meas.buzz, "goo")
    assert exp == TagExp(meas.buzz, GT, "goo")


def test_less_than():
    meas = Measurement.new("fizz")
    exp = TagExp.less_than(meas.buzz, "goo")
    assert exp == TagExp(meas.buzz, LT, "goo")


def test_greater_equal():
    meas = Measurement.new("fizz")
    exp = TagExp.greater_equal(meas.buzz, "goo")
    assert exp == TagExp(meas.buzz, GE, "goo")


def test_less_equal():
    meas = Measurement.new("fizz")
    exp = TagExp.less_equal(meas.buzz, "goo")
    assert exp == TagExp(meas.buzz, LE, "goo")


def test_like():
    meas = Measurement.new("fizz")
    exp = TagExp.like(meas.buzz, "goo")
    assert exp == TagExp(meas.buzz, LK, "goo")


def test_notlike():
    meas = Measurement.new("fizz")
    exp = TagExp.notlike(meas.buzz, "goo")
    assert exp == TagExp(meas.buzz, NK, "goo")



""" InfluxAlchemy client tests. """

import mock
import influxdb
from influxalchemy.client import InfluxAlchemy
from influxalchemy.measurement import Measurement
from influxalchemy.query import InfluxDBQuery


@mock.patch("influxdb.InfluxDBClient")
def test_query(mock_flux):
    db = influxdb.InfluxDBClient(database="fizz")
    db.query.side_effect = influxdb.exceptions.InfluxDBClientError(None)
    client = InfluxAlchemy(db)
    query = client.query(Measurement.new("buzz"))
    assert str(query) == "SELECT * FROM buzz;"


@mock.patch("influxdb.InfluxDBClient.query")
def test_measurements(mock_flux):
    mock_res = mock.MagicMock()
    mock_res.get_points.return_value = [{"name": "fizz"}]
    mock_flux.return_value = mock_res
    db = influxdb.InfluxDBClient(database="fizz")
    client = InfluxAlchemy(db)
    measurements = list(client.measurements())
    mock_flux.assert_called_once_with("SHOW MEASUREMENTS;")


@mock.patch("influxdb.InfluxDBClient.query")
def test_tags(mock_flux):
    mock_res = mock.MagicMock()
    mock_res.get_points.return_value = [{'tagKey': 'sensor_id'}]
    mock_flux.return_value = mock_res
    db = influxdb.InfluxDBClient(database="fizz")
    client = InfluxAlchemy(db)
    assert client.tags(Measurement.new("environment")) == ["sensor_id"]


@mock.patch("influxdb.InfluxDBClient.query")
def test_fields(mock_flux):
    mock_res = mock.MagicMock()
    mock_res.get_points.return_value = [
        {'fieldKey': 'humidity', 'fieldType': 'float'},
        {'fieldKey': 'temperature', 'fieldType': 'float'}
    ]
    mock_flux.return_value = mock_res
    db = influxdb.InfluxDBClient(database="fizz")
    client = InfluxAlchemy(db)
    assert client.fields(Measurement.new("environment")) == ["humidity", "temperature"]



import textwrap
from setuptools import setup

setup(
    author='amancevice',
    author_email='smallweirdnum@gmail.com',
    description='Interact with InfluxDB using SQLAlchemy-style syntax',
    install_requires=[
        'influxdb >= 5.0.0',
        'pytz >= 2018.3',
        'requests >= 2.20.0',
        'six >= 1.11.0',
    ],
    name='influxalchemy',
    packages=['influxalchemy'],
    setup_requires=['setuptools_scm'],
    url='https://github.com/amancevice/influxalchemy',
    use_scm_version=True,
)
